
#include "EquazioneRetta.h"
//---------------------------------------------------------------------------
// costruttore mediante due punti
EquazioneRetta::EquazioneRetta(double x1,double y1,double x2,double y2) {
		double m,t; 	// coefficienti
		double dx,dy;

		dx = x2-x1;
		dy = y2-y1;

		if (dx == 0) {
			// retta verticale
			a = 1;
			b = 0;
			c = -x1;
		} else if (dy == 0) {
			// retta orizzontale
			a = 0;
			b = 1;
			c = -y1;
		} else if (dx >= dy) {
			m = dy / dx;	// coefficiente angolare

			a = m;
			b = -1;
			c = -m * x1 + y1;
		} else {
			t = dx / dy;	// inverso del coefficiente angolare

			a = 1;
			b = -t;
			c = - x1 + t*y1;
		} // endif
	}

	// costruttore della classe
EquazioneRetta::EquazioneRetta(double aa,double bb,double cc) {
	a = aa;
    b = bb;
    c = cc;
}

double EquazioneRetta::X(double y) {
	if (a == 0)
    	throw 100;	// errore retta orizontale

    return (- (b*y + c) / a);
}

	// fornisce la Y data la X
	// pu generare eccezzioni in caso di retta verticale
double EquazioneRetta::Y(double x) {
	if (b == 0)
    	throw 101;   // errore: retta verticale

    return (- (a*x + c) / b);
}

// ritorna il coefficente angolare della retta
double EquazioneRetta::coefficente_angolare() {
	double m;

    if ( b != 0)	// se la retta non  verticale
    	m = -a/b;
    else
    	m = 1.7E308 ; // MAX_DOUBLE

    return m;
}

Angolo EquazioneRetta::angolo() {
	double m = coefficente_angolare();
    double alfa;
    Angolo temp;

    alfa = atan(m);

    temp.radianti(alfa);

    return temp;
}

bool EquazioneRetta::coincidenti(EquazioneRetta r1, EquazioneRetta r2) {
		bool test1,test2=false;

		test1 = (r1.coefficente_angolare() == r2.coefficente_angolare());

		try {
			if (r1.verticale())
				test2 = (r1.X(0) == r2.X(0));
			else if (r1.orizzontale())
				test2 = (r1.Y(0) == r2.Y(0));
			else {
				if (r1.coefficente_angolare() < 1)
					test2 = (r1.Y(0) == r2.Y(0));
				else
					test2 = (r1.X(0) == r2.X(0));
			}
		} catch (...) {
		}
		// coefficienti angolari uguali
		return test1 && test2;
	}
	
	// determina se due rette sono parallele
bool EquazioneRetta::parallele(EquazioneRetta r1,EquazioneRetta r2) {
		double m1,m2;
		
		m1 = r1.coefficente_angolare();
		m2 = r2.coefficente_angolare();
		
		return (m1 == m2);
	}

	// risolve il sistema formato dalle equazioni delle due rette
	// determinando quella coppia (x,y) comune alle due rette
DCoord EquazioneRetta::intersezione(EquazioneRetta r1,EquazioneRetta r2) {
		DCoord I;
		double Arapp;
		double x,y;
		
		if (parallele(r1,r2))
			throw 102;    	// errore rette parallele: nessuna intersezione
		
		if (r1.orizzontale()) {	// casi PARTICOLARI
			// r1  orizzontale...
			y = r1.Y(0);
			x = r2.X(y);
		} else if (r1.verticale()) {
			x = r1.X(0);
			y = r2.Y(x);
		} else if (r2.orizzontale()) {
			// r1  orizzontale...
			y = r2.Y(0);
			x = r1.X(y);
		} else if (r2.verticale()) {
			x = r2.X(0);
			y = r1.Y(x);
		} else if (r2.a != 0) { 	// Casi GENERALI
			Arapp = r1.a / r2.a;
			
			y = (Arapp * r2.c - r1.c) / (r1.b - Arapp * r2.b);
			x = r2.X(y);
		} else {
			Arapp =  r2.a / r1.a;
		
			y = (Arapp * r1.c - r2.c) / (r2.b - Arapp * r1.b);
			x = r1.X(y);
		}
		
		I.x = x;
		I.y = y;
		
		return I;
	}
		
