#include "Grafo.h"
//---------------------------------------------------------------------------

grafo::grafo(ifstream &canale) {
	leggi_da_file(canale);

}

grafo::~grafo() {

    for (int i = 0; i<links.size; i++)
        delete [] segmenti[i];

    delete[] segmenti;

	delete[] nodes.element;
    delete[] links.element;
}

// carica il grafo da un file descrittore
// in formato NETFORMAT:
// num1 num2	(num1 = numero nodi totali, num2 = numero coll/nodo)
// node coll coll1 peso1 coll2 peso2 ...
void grafo::leggi_da_file(ifstream &canale) {
	int nodo,legame;
    int link,w;
    int i;

	canale >> nodes.size;		// numero di nodi
    canale >> statistica;   	// legami per nodo (massimo)

    links.sizeMax = nodes.size * statistica;	// massimo numero di legami possibili

    nodes.element = new Nodo[nodes.size];
    links.element = new Legame[links.sizeMax];
    links.size = 0;

    do {
    	canale >> nodo;		// numero del nodo
        canale >> legame;   // legami per nodo
        nodes.element[nodo].tipo = legame;

        for (i = 0; i<legame; i++) {
          	canale >> link;		// legame
            canale >> w;        // peso del legame

            if (!links.esiste(nodo,link)) {
            	links.element[links.size].source = nodo;
                links.element[links.size].dest = link;
                links.element[links.size].peso = w;
                links.size ++;
            }
        }
    } while (nodo < nodes.size-1);

}

void grafo::scrivi_su_file(ofstream &canale) {
    int i,j;
    int adj;
    Nodo n;

	canale << nodes.size << " " << statistica << endl;

    for (i = 0; i<nodes.size; i++) {
    	n = nodes.element[i];

        canale << i << " ";
        adj = 0;

        // conta le adiacenze
        for (j = 0; j<links.size; j++)
        	if (links.element[j].contiene(i))
            	adj++;

        canale << adj << " ";

        // scrive le adiacenze
        for (j = 0; j<links.size; j++)
        	if (links.element[j].contiene(i)) {
            	if (links.element[j].source == i)
            		canale << links.element[j].dest << " ";
                else
                	canale << links.element[j].source << " ";

            	canale << links.element[j].peso << " " ;
            }

        canale << endl;
    }

    canale << endl;

    for (i = 0; i<nodes.size; i++) {
    	canale << i << " " << nodes.element[i].A.x;
        canale << " " << nodes.element[i].A.y << " ";
        canale << nodes.element[i].tipo << " 1 '";
        canale << "label " << i << "' " << endl;

    }
}

void grafo::disponi_in_disordine() {
    Legame l;
    Nodo n1,n2;

    int i;
    int x,y;

    // la griglia ha come lato il doppio della radice del numero di nodi.
    // cio pu ospitare il doppio dei nodi presenti
    lato = (int) floor ( sqrt(nodes.size) * 2.0 + 0.5);

    // reset
    for (i = 0; i<nodes.size; i++) {
    	nodes.element[i].A.x = -1;
    	nodes.element[i].A.y = -1;
    }

    // assegna nuove posizioni
    for (i = 0; i<nodes.size; i++) {

        // trova una posizione libera
    	do {
        	x = casuale(lato);
            y = casuale(lato);
        } while (nodes.occupato(x,y));

        nodes.element[i].A.x = x;
        nodes.element[i].A.y = y;
    }


    segmenti = new Segmento*[links.size];

    // prepara la lista di Segmenti
    for (int i = 0; i<links.size; i++) {
        l = links.element[i];
        n1 = nodes.element[l.source];
        n2 = nodes.element[l.dest];

        segmenti[i] = new Segmento(n1.A.x,n1.A.y,n2.A.x,n2.A.y);
    }

}

int grafo::intersezioni() {
    Segmento *s1,*s2;
    int i,j;
    int contatore = 0;

    for (i = 0; i<links.size; i++) {
    	s1 = segmenti[i];

        for (j = i+1; j<links.size; j++) {
        	s2 = segmenti[j];

            if (!Segmento::nodo_comune(*s1,*s2))
            	if( Segmento::intersezione(*s1,*s2) )
                	contatore++;
        }
    }

    return contatore;
}

int grafo::conflitti() {
    Segmento *s1;
    Nodo n;
    Legame l;
    DCoord A;
    int i,j;
    int contatore = 0;

    for (i = 0; i<links.size; i++) {
    	s1 = segmenti[i];
        l = links.element[i];

        for (j = 0; j<nodes.size; j++)
        	if (!l.contiene(j)) {
            	n = nodes.element[j];
                A.x = n.A.x;
                A.y = n.A.y;

                if (s1->distanza_punto(A,0.3))
                	contatore++;
            }
    }

    return contatore;
}

int grafo::intersezioni_legame(int indice) {
    Segmento *s1,*s2;
    int j;
    int contatore = 0;

    s1 = segmenti[indice];

    for (j = 0; j<links.size; j++) {
    	if (j != indice) {
        	s2 = segmenti[j];

            if (!Segmento::nodo_comune(*s1,*s2))
            	if( Segmento::intersezione(*s1,*s2) )
                    contatore++;
        }
    }

    return contatore;
}

int grafo::conflitti_legame(int indice) {
    Segmento *s1;
    Nodo n;
    Legame l;
    DCoord A;
    int j;
    int contatore = 0;

    s1 = segmenti[indice];
    l = links.element[indice];

    for (j = 0; j<nodes.size; j++)
    	if (!l.contiene(j)) {
        	n = nodes.element[j];
            A.x = n.A.x;
            A.y = n.A.y;

            if (s1->distanza_punto(A,0.3))
            	contatore++;
        }

    return contatore;
}

// effettua una mossa casuale, ovvero uno spostamento di un nodo da una
// posizione, ad un'altra. Deve ricalcolare i segmenti che in questa maniera
// variano. Fornisce in risposta la variazione sul numero di intersezioni.
int grafo::genera_mossa_casuale() {
	int indice,pox,poy;
    int countPrima,countDopo;
    int i;


    indice = casuale(nodes.size-1);	// sceglie un nodo

    pox = casuale(lato);	// genera una nuova coppia di coordinate
    poy = casuale(lato);

    // bisogna contare il numero di intersezioni implicate PRIMA
    countPrima = 0;
    for (i = 0; i<links.size; i++)
    	if (links.element[i].contiene(indice))
        	countPrima += intersezioni_legame(i) + conflitti_legame(i);

    // memorizza le informazioni necessarie all'eventuale ripristino
    memIndice = indice;
    memX = nodes.element[indice].A.x;
    memY = nodes.element[indice].A.y;

    // effettua lo spostamento
    muovi_nodo(indice,pox,poy);

    // bisogna contare il numero di intersezioni implicate DOPO
    countDopo = 0;
    for (i = 0; i<links.size; i++)
    	if (links.element[i].contiene(indice))
        	countDopo += intersezioni_legame(i) + conflitti_legame(i);

    return countDopo-countPrima;
}

void grafo::annulla_mossa() {

    // effettua lo spostamento
    muovi_nodo(memIndice,memX,memY);

}

void grafo::muovi_nodo(int indice,int pox,int poy) {
	DCoord P;
    int i;
    Segmento *sOld,*sNew;

    P.x = pox;
    P.y = poy;

	if (!nodes.occupato(pox,poy)) {
    	nodes.element[indice].A.x = pox;
    	nodes.element[indice].A.y = poy;

        // tutti i segmenti che hanno come nodo 'indice' vanno modificati
    	for (i = 0; i<links.size; i++)
        	if (links.element[i].contiene(indice)) {
            	sOld = segmenti[i];	// preleva il segmento

                // ottiene una versione 'modificata'
                if (links.element[i].source == indice)
                	sNew = Segmento::modifica_inizio(*sOld,P);
                else
                	sNew = Segmento::modifica_fine(*sOld,P);

                // elimina quello vecchio e vi sostituisce quello nuovo
                delete sOld;
                segmenti[i] = sNew;
            }
    }
}



