
#include "Segmento.h"
//---------------------------------------------------------------------------

Segmento::Segmento(double x1,double y1,double x2,double y2)
			: EquazioneRetta(x1,y1,x2,y2) {
	inizio.x = x1;
    inizio.y = y1;
    fine.x = x2;
    fine.y = y2;

    if (x1 < x2) {
    	minx = x1;
        maxx = x2;
    } else {
    	minx = x2;
        maxx = x1;
    }

    if (y1 < y2) {
    	miny = y1;
        maxy = y2;
    } else {
    	miny = y2;
        maxy = y1;
    }

}

Segmento::Segmento(DCoord A, DCoord B)
			: EquazioneRetta(A.x,A.y,B.x,B.y) {
	inizio = A;
    fine = B;

    if (A.x < B.x) {
    	minx = A.x;
        maxx = B.x;
    } else {
    	minx = B.x;
        maxx = A.x;
    }

    if (A.y < B.y) {
    	miny = A.y;
        maxy = B.y;
    } else {
    	miny = B.y;
        maxy = A.y;
    }
}

bool Segmento::xRange(double x) {
  	return (x >= minx && x <= maxx);
}

bool Segmento::yRange(double y) {
  	return (y >= miny && y <= maxy);
}

bool Segmento::Range(DCoord A) {
  	return xRange(A.x) && yRange(A.y);
}

// escluso gli estremi...
bool Segmento::contiene(DCoord A) {
  	bool test1;
    if (minx != maxx && miny != maxy)
     	test1 = (A.x > minx && A.x < maxx) && (A.y > miny && A.y < maxy);
    else if (minx != maxx)
    	test1 = (A.x > minx && A.x < maxx) && (A.y == miny);
    else if (miny != maxy)
    	test1 = (A.x == minx) && (A.y > miny && A.y < maxy);

    bool test2 = false;
    double x,y;

    if (test1) {
    	if (coefficente_angolare() < 1) {
        	// caso retta orizontale
            y = Y(A.x);
    		test2 = (floor(y+0.5) == floor(A.y+0.5));
        } else {
        	// caso retta verticale
        	x = X(A.y);
            test2 = (floor(x+0.5) == floor(A.x+0.5));
        }
    }

    return test2;
}

DCoord Segmento::preleva_inizio() {
	return inizio;
}

DCoord Segmento::preleva_fine() {
	return fine;
}


bool Segmento::nodo_comune(Segmento s1,Segmento s2) {
	return ( s1.inizio == s2.inizio || s1.inizio == s2.fine) ||
    	   ( s1.fine == s2.inizio || s1.fine == s2.fine);
}

Segmento *Segmento::modifica_inizio(Segmento s, DCoord A) {
	return new Segmento(A,s.fine);
}

Segmento *Segmento::modifica_fine(Segmento s, DCoord A) {
	return new Segmento(s.inizio,A);
}

bool Segmento::intersezione(Segmento s1,Segmento s2) {
	DCoord I;
    bool test1,test2;

    if (!EquazioneRetta::parallele(s1,s2)) {
    	I = EquazioneRetta::intersezione(s1,s2);

    	test1 = s1.contiene(I);
    	test2 = s2.contiene(I);

    	if (!test1 || !test2)
    		return false;		// punto esterno
    } else {
    	return false;
    }

    return true;
}

bool Segmento::distanza_punto(DCoord punto,double MAX) {
	EquazioneRetta *orto;
    DCoord I;;
    double coeff1,coeff2;
    double a1,b1,c1;
    double distanza;

    if (orizzontale()) {
    	a1 = 1;
        b1 = 0;
        c1 = -punto.x;
    } else if (verticale()) {
    	a1 = 0;
        b1 = 1;
        c1 = -punto.y;
    } else {
    	coeff1 = coefficente_angolare();
        coeff2 = - 1.0/coeff1;

        a1 = coeff2;
        b1 = -1;
        c1 = punto.y - coeff2 * punto.x;
    }

    orto = new EquazioneRetta(a1,b1,c1);

    // una retta e la sua ortogonale DEVONO avere l'intersezione
    I = EquazioneRetta::intersezione(*this,*orto);

    if (Range(I)) {
    	distanza = DCoord::distanza_euclidea(punto,I);
        return (distanza <= MAX);
    }

    return false;
}
		



